#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define NKEYS 12
#define STRMAX 1024

/* ========================================================================== *
 * wrap
 *
 * Program to read objects from one MIF file,
 * shift them 360 map units east, and write to another MIF file.
 *
 * Usage: wrap <input> <output>
 *
 * No actions on MID file are carried out,
 * just use original one copying or renaming.
 * -------------------------------------------------------------------------- */
int main(int argc, char *argv[])
{
  char buf[STRMAX], w[8][128];
  char *keyword[NKEYS] = {
    "Arc", "Ellipse", "Line", "Pline", "Point", "Region", "Rect",
    "Roundrect", "Text", "Multipoint", "Center", "Multiple"
  };
  double x, x2;
  int nsec, npt, t, k;
  FILE *fp0, *fp1;

  if (argc < 3) {
    fputs("usage: wrap <input> <output>\n", stderr);
    exit(EXIT_SUCCESS);
  }
  if ((fp0 = fopen(argv[1], "r")) == NULL) {
    fprintf(stderr, "can't open %s\n", argv[1]);
    exit(EXIT_FAILURE);
  }
  if ((fp1 = fopen(argv[2], "w")) == NULL) {
    fprintf(stderr, "can't open %s\n", argv[2]);
    exit(EXIT_FAILURE);
  }
  nsec = npt = t = 0;
  while (fgets(buf, STRMAX, fp0) != NULL) {
    if (npt == 0) {
      if (nsec == 0) {
	switch (t) {
	case 2: /* Text string */
	  fputs(buf, fp1);
	  t = 1;
	  break;
	case 1: /* Text bbox */
	  sscanf(buf, "%s %s %s %s", w[0], w[1], w[2], w[3]);
	  x = atof(w[0]) + 360.0;
	  x2 = atof(w[2]) + 360.0;
	  fprintf(fp1, "    %f %s %f %s\n", x, w[1], x2, w[3]);
	  t = 0;
	  break;
	default:
	  sscanf(buf, "%s %s %s %s %s %s %s",
		 w[0], w[1], w[2], w[3], w[4], w[5], w[6]);
	  for (k = 0; k < NKEYS; k++) {
	    if (strcmp(w[0], keyword[k]) == 0)
	      break;
	  }
	  switch (k) {
	  case 8: /* "Text" */
	    t = 2;
	    fputs(buf, fp1);
	    break;
	  case 9: /* "Multipoint" */
	    npt = atoi(w[1]);
	    fputs(buf, fp1);
	    break;
	  case 5: /* "Region" */
	    nsec = atoi(w[1]);
	    fputs(buf, fp1);
	    break;
	  case 3: /* "Pline" */
	    if (strcmp(w[1], keyword[11]) == 0) /* "Pline Multiple" */
	      nsec = atoi(w[2]);
	    else
	      npt = atoi(w[1]);
	    fputs(buf, fp1);
	    break;
	  case 4: /* "Point" */
	    x = atof(w[1]) + 360.0;
	    fprintf(fp1, "%s %f %s\n", w[0], x, w[2]);
	    break;
	  case 10: /* "Center" */
	    x = atof(w[1]) + 360.0;
	    fprintf(fp1, "    %s %f %s\n", w[0], x, w[2]);
	    break;
	  case 1: /* "Ellipse" */
	  case 2: /* "Line" */
	  case 6: /* "Rect" */
	    x = atof(w[1]) + 360.0;
	    x2 = atof(w[3]) + 360.0;
	    fprintf(fp1, "%s %f %s %f %s\n",
		    w[0], x, w[2], x2, w[4]);
	    break;
	  case 7: /* "Roundrect" */
	    x = atof(w[1]) + 360.0;
	    x2 = atof(w[3]) + 360.0;
	    fprintf(fp1, "%s %f %s %f %s %s\n",
		    w[0], x, w[2], x2, w[4], w[5]);
	    break;
	  case 0: /* "Arc" */
	    x = atof(w[1]) + 360.0;
	    x2 = atof(w[3]) + 360.0;
	    fprintf(fp1, "%s %f %s %f %s %s %s\n",
		    w[0], x, w[2], x2, w[4], w[5], w[6]);
	    break;
	  default:
	    fputs(buf, fp1);
	  }
	}
      } else {
	npt = atoi(buf);
	fputs(buf, fp1);
	nsec--;
      }
    } else {
      sscanf(buf, "%s %s", w[0], w[1]);
      x = atof(w[0]) + 360.0;
      fprintf(fp1, "%f %s\n", x, w[1]);
      npt--;
    }
  }
  fclose(fp1);
  fclose(fp0);
  return 0;
}
/* ========================================================================== */
